import numpy as np


class KMeans(object):
    
    def __init__(self, k, dim, cookbook=None, n_iterations=0):
        self.k = k
        self.dim = dim
        if cookbook is None:
            self.cookbook = np.random.randn(k, dim) * 0.1
        else:
            self.cookbook = cookbook
        self.n_iterations = n_iterations
        
    def learn(self, data):
        new_cookbook = k_means_learn(data, self.cookbook)
        return KMeans(self.k, self.dim,
                      new_cookbook, self.n_iterations+1)


def k_means_classify(data, cookbook):
    
    indices = np.empty((data.shape[0],), dtype=int)
    
    for row_idx, row in enumerate(data):
        
        # Look for cookbook entry with shortest distance.
        min_dist = np.inf
        closest_idx = -1
        for vect_idx, vect in enumerate(cookbook):
            dist = np.linalg.norm(row-vect, 2)
            if dist < min_dist:
                min_dist = dist
                closest_idx = vect_idx
    
        indices[row_idx] = closest_idx

    return indices


def k_means_learn(data, cookbook):
    
    indices = k_means_classify(data, cookbook)
    
    new_cookbook = np.empty_like(cookbook)
    for idx in xrange(cookbook.shape[0]):
        cluster = data[indices==idx,:]
        new_cookbook[idx,:] = cluster.mean(axis=0)
    
    return new_cookbook
