import unittest
import numpy as np
from numpy.testing import assert_array_equal
from k_means import k_means_classify, k_means_learn


class TestKMeans(unittest.TestCase):
    
    def test_k_means_classify(self):
        # Given
        cookbook = np.array(
            [[1.0,  1.0],
             [1.0, -1.0]]
        )

        n = 1000
        data = np.random.randn(n, 2)

        # When
        indices = k_means_classify(data, cookbook)

        # Then
        expected = np.where(data[:,1] > 0.0, 0, 1)
        assert_array_equal(indices, expected)

    def test_k_means_learn(self):
        # Given
        data = np.array(
            [[0.0,  1.0],
             [0.0, -1.0]]
        )
        
        cookbook = np.random.randn(2, 2) * 0.1

        # When
        n_iterations = 100
        for t in xrange(n_iterations):
            # Another time: immutable data structure?
            cookbook = k_means_learn(data, cookbook)

        # Then
        expected = np.array(
            [[0.0,  1.0],
             [0.0, -1.0]]
        )
        
        for result in cookbook:
            closest_dist = min(
                np.linalg.norm(result-expected[0,:], 2),
                np.linalg.norm(result-expected[1,:], 2)
            )
            self.assertAlmostEqual(closest_dist, 0.0, 100)

        # Discuss: sometimes this test will fail, as both
        # data points will be assigned to a single point.
        # A possible initialization method is to pick random
        # points from the data set.


if __name__ == '__main__':
    unittest.main()
